@@ -5,19 +5,31 @@
 """Two-key triple DES cipher (also known as DES3, 3DES, Triple DES,
 or DES-EDE)."""
 
+import sys
+
+if sys.version_info < (3, 8):
+    from typing_extensions import Final, Literal
+else:
+    from typing import Final, Literal
+
+from typing import Optional, Union
 
 from mbedtls.exceptions import TLSError
 
-from . import _cipher
+from ._cipher import Cipher, Mode
 
 __all__ = ["block_size", "key_size", "new"]
 
 
-block_size = 8
-key_size = 16
+block_size: Final = 8
+key_size: Final = 16
 
 
-def new(key, mode, iv=None):
+def new(
+    key: bytes,
+    mode: Union[int, Literal[Mode.CBC, Mode.ECB]],
+    iv: Optional[bytes] = None,
+) -> Cipher:
     """Return a `Cipher` object that can perform two-key triple DES
     encryption and decryption.
 
@@ -25,21 +37,21 @@ def new(key, mode, iv=None):
     or DES-EDE).
 
     Parameters:
-        key (bytes or None): The key to encrypt decrypt.  If None,
+        key: The key to encrypt decrypt.  If None,
             encryption and decryption are unavailable.
-        mode (Mode): The mode of operation of the cipher.
-        iv (bytes or None): The initialization vector (IV).  The IV is
+        mode: The mode of operation of the cipher.
+        iv: The initialization vector (IV).  The IV is
             required for every mode but ECB and CTR where it is ignored.
             If not set, the IV is initialized to all 0, which should not
             be used for encryption.
 
     """
-    mode = _cipher.Mode(mode)
+    mode_ = Mode(mode)
     if len(key) != key_size:
         raise TLSError(
             msg="key size must be %i bytes, got %i" % (key_size, len(key))
         )
-    if mode not in {_cipher.Mode.ECB, _cipher.Mode.CBC}:
-        raise TLSError(msg="unsupported mode %r" % mode)
-    name = ("DES-EDE-%s" % mode.name).encode("ascii")
-    return _cipher.Cipher(name, key, mode, iv)
+    if mode_ not in {Mode.CBC, Mode.ECB}:
+        raise TLSError(msg="unsupported mode %r" % mode_)
+    name = ("DES-EDE-%s" % mode_.name).encode("ascii")
+    return Cipher(name, key, mode_, iv)
