@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import datetime as dt
 import pickle
 import socket
@@ -6,13 +8,23 @@ import sys
 import time
 from contextlib import suppress
 from pathlib import Path
+from typing import (
+    Any,
+    Callable,
+    Iterator,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 import pytest
 
 from mbedtls import hashlib
-from mbedtls._tls import _DTLSCookie as DTLSCookie
-from mbedtls._tls import _PSKSToreProxy as PSKStoreProxy
-from mbedtls.pk import RSA
+from mbedtls._tls import _DTLSCookie as DTLSCookie  # type: ignore
+from mbedtls._tls import _PSKSToreProxy as PSKStoreProxy  # type: ignore
+from mbedtls.pk import ECC, RSA
 from mbedtls.tls import (
     ClientContext,
     DTLSConfiguration,
@@ -35,20 +47,23 @@ from mbedtls.tls import (
 )
 from mbedtls.x509 import CRT, CSR, BasicConstraints
 
+_Key = Union[RSA, ECC]
+_HostName = str
+
 
 @pytest.fixture(scope="module")
-def rootpath():
+def rootpath() -> Path:
     return Path(__file__).parent.parent
 
 
 def make_root_ca(
-    subject=None,
-    not_before=None,
-    not_after=None,
-    serial_number=None,
-    basic_constraints=None,
-    digestmod=None,
-):
+    subject: Optional[str] = None,
+    not_before: Optional[dt.datetime] = None,
+    not_after: Optional[dt.datetime] = None,
+    serial_number: Optional[int] = None,
+    basic_constraints: Optional[BasicConstraints] = None,
+    digestmod: Optional[hashlib.Algorithm] = None,
+) -> Tuple[CRT, _Key]:
     if subject is None:
         subject = "OU=test, CN=Trusted CA"
     if not_before is None:
@@ -76,15 +91,15 @@ def make_root_ca(
 
 
 def make_crt(
-    issuer_crt,
-    issuer_key,
-    subject=None,
-    not_before=None,
-    not_after=None,
-    serial_number=None,
-    basic_constraints=None,
-    digestmod=None,
-):
+    issuer_crt: CRT,
+    issuer_key: _Key,
+    subject: Optional[str] = None,
+    not_before: Optional[dt.datetime] = None,
+    not_after: Optional[dt.datetime] = None,
+    serial_number: Optional[int] = None,
+    basic_constraints: Optional[BasicConstraints] = None,
+    digestmod: Optional[hashlib.Algorithm] = None,
+) -> Tuple[CRT, _Key]:
     if subject is None:
         subject = "OU=test, CN=hostname"
     if not_before is None:
@@ -115,22 +130,22 @@ def make_crt(
 class TestPickle:
     @pytest.mark.parametrize(
         "obj",
-        (
+        [
             TLSConfiguration(),
             DTLSConfiguration(),
             ClientContext(TLSConfiguration()),
             ClientContext(DTLSConfiguration()),
             ServerContext(TLSConfiguration()),
             ServerContext(DTLSConfiguration()),
-        ),
+        ],
         ids=type,
     )
-    def test_picklable(self, obj):
+    def test_picklable(self, obj: object) -> None:
         assert obj == pickle.loads(pickle.dumps(obj))
 
     @pytest.mark.parametrize(
         "obj",
-        (
+        [
             TLSSession(),
             TLSWrappedBuffer(ClientContext(DTLSConfiguration())),
             TLSWrappedBuffer(ClientContext(TLSConfiguration())),
@@ -152,10 +167,10 @@ class TestPickle:
                 socket.socket(),
                 TLSWrappedBuffer(ServerContext(TLSConfiguration())),
             ),
-        ),
+        ],
         ids=type,
     )
-    def test_unpicklable(self, obj):
+    def test_unpicklable(self, obj: object) -> None:
         with pytest.raises(TypeError) as excinfo:
             pickle.dumps(obj)
 
@@ -163,42 +178,54 @@ class TestPickle:
 
 
 class TestPSKStoreProxy:
-    @pytest.fixture
-    def psk_store(self):
+    @pytest.fixture()
+    def psk_store(self) -> Mapping[str, bytes]:
         return {"client": b"the secret key"}
 
-    @pytest.fixture
-    def proxy(self, psk_store):
+    @pytest.fixture()
+    def proxy(self, psk_store: Mapping[str, bytes]) -> PSKStoreProxy:
         return PSKStoreProxy(psk_store)
 
-    @pytest.mark.parametrize("repr_", (repr, str), ids=lambda f: f.__name__)
-    def test_repr(self, repr_, psk_store):
+    @pytest.mark.parametrize(
+        "repr_",
+        [repr, str],
+        ids=lambda f: f.__name__,  # type: ignore[no-any-return]
+    )
+    def test_repr(
+        self, repr_: Callable[[object], str], psk_store: Mapping[str, bytes]
+    ) -> None:
         assert isinstance(repr_(psk_store), str)
 
-    def test_unwrap(self, proxy, psk_store):
+    def test_unwrap(
+        self, proxy: PSKStoreProxy, psk_store: Mapping[str, bytes]
+    ) -> None:
         assert proxy.unwrap() == psk_store
 
-    def test_eq(self, proxy, psk_store):
+    def test_eq(
+        self, proxy: PSKStoreProxy, psk_store: Mapping[str, bytes]
+    ) -> None:
         for k, v in psk_store.items():
             assert proxy[k] == v
 
-    def test_len(self, proxy, psk_store):
+    def test_len(
+        self, proxy: PSKStoreProxy, psk_store: Mapping[str, bytes]
+    ) -> None:
         assert len(proxy) == len(psk_store)
 
 
 class TestTLSVersion:
     @pytest.mark.parametrize("version", TLSVersion)
-    def test_major(self, version):
+    def test_major(self, version: TLSVersion) -> None:
         assert version.major() == 3
 
-    def test_minor(self):
+    def test_minor(self) -> None:
         # assert TLSVersion.SSLv3.minor() == 0
         assert TLSVersion.TLSv1.minor() == 1
         assert TLSVersion.TLSv1_1.minor() == 2
         assert TLSVersion.TLSv1_2.minor() == 3
 
     @pytest.mark.parametrize("version", TLSVersion)
-    def test_from_major_minor(self, version):
+    def test_from_major_minor(self, version: TLSVersion) -> None:
         assert (
             TLSVersion.from_major_minor(version.major(), version.minor())
             is version
@@ -207,21 +234,21 @@ class TestTLSVersion:
     @pytest.mark.parametrize(
         "version", [TLSVersion.MINIMUM_SUPPORTED, TLSVersion.MAXIMUM_SUPPORTED]
     )
-    def test_minmax_supported(self, version):
+    def test_minmax_supported(self, version: TLSVersion) -> None:
         assert version in TLSVersion
 
 
 class TestDTLSVersion:
     @pytest.mark.parametrize("version", DTLSVersion)
-    def test_major(self, version):
+    def test_major(self, version: DTLSVersion) -> None:
         assert version.major() == 3
 
-    def test_minor(self):
+    def test_minor(self) -> None:
         assert DTLSVersion.DTLSv1_0.minor() == 2
         assert DTLSVersion.DTLSv1_2.minor() == 3
 
     @pytest.mark.parametrize("version", DTLSVersion)
-    def test_from_major_minor(self, version):
+    def test_from_major_minor(self, version: DTLSVersion) -> None:
         assert (
             DTLSVersion.from_major_minor(version.major(), version.minor())
             is version
@@ -231,41 +258,62 @@ class TestDTLSVersion:
         "version",
         [DTLSVersion.MINIMUM_SUPPORTED, DTLSVersion.MAXIMUM_SUPPORTED],
     )
-    def test_minmax_supported(self, version):
+    def test_minmax_supported(self, version: DTLSVersion) -> None:
         assert version in DTLSVersion
 
 
 class TestTLSRecordHeader:
     @pytest.fixture(params=TLSRecordHeader.RecordType)
-    def record_type(self, request):
+    def record_type(self, request: Any) -> TLSRecordHeader.RecordType:
+        assert isinstance(request.param, TLSRecordHeader.RecordType)
         return request.param
 
     @pytest.fixture(params=TLSVersion)
-    def version(self, request):
+    def version(self, request: Any) -> TLSVersion:
+        assert isinstance(request.param, TLSVersion)
         return request.param
 
-    @pytest.fixture
-    def length(self):
+    @pytest.fixture()
+    def length(self) -> int:
         return 42
 
-    @pytest.fixture
-    def header(self, record_type, version, length):
+    @pytest.fixture()
+    def header(
+        self,
+        record_type: TLSRecordHeader.RecordType,
+        version: TLSVersion,
+        length: int,
+    ) -> TLSRecordHeader:
         return TLSRecordHeader(record_type, version, length)
 
-    @pytest.mark.parametrize("repr_", (repr, str), ids=lambda f: f.__name__)
-    def test_repr(self, repr_, record_type):
+    @pytest.mark.parametrize(
+        "repr_",
+        [repr, str],
+        ids=lambda f: f.__name__,  # type: ignore[no-any-return]
+    )
+    def test_repr(
+        self,
+        repr_: Callable[[object], str],
+        record_type: TLSRecordHeader.RecordType,
+    ) -> None:
         assert isinstance(repr_(record_type), str)
 
-    def test_hash(self, record_type):
+    def test_hash(self, record_type: TLSRecordHeader.RecordType) -> None:
         assert isinstance(hash(record_type), int)
 
-    def test_accessors(self, header, record_type, version, length):
+    def test_accessors(
+        self,
+        header: TLSRecordHeader,
+        record_type: TLSRecordHeader.RecordType,
+        version: TLSVersion,
+        length: int,
+    ) -> None:
         assert len(header) == 5
         assert header.record_type is record_type
         assert header.version is version
         assert header.length == length
 
-    def test_serialization(self, header):
+    def test_serialization(self, header: TLSRecordHeader) -> None:
         serialized = bytes(header)
         assert isinstance(serialized, bytes)
         assert len(serialized) == 5
@@ -273,47 +321,53 @@ class TestTLSRecordHeader:
 
 
 class TestTLSSession:
-    @pytest.fixture
-    def session(self):
+    @pytest.fixture()
+    def session(self) -> TLSSession:
         return TLSSession()
 
-    def test_repr(self, session):
+    def test_repr(self, session: TLSSession) -> None:
         assert isinstance(repr(session), str)
 
 
 class TestTrustStore:
-    @pytest.fixture
-    def store(self):
+    @pytest.fixture()
+    def store(self) -> TrustStore:
         return TrustStore.system()
 
-    @pytest.mark.parametrize("repr_", (repr, str), ids=lambda f: f.__name__)
-    def test_repr(self, repr_, store):
+    @pytest.mark.parametrize(
+        "repr_",
+        [repr, str],
+        ids=lambda f: f.__name__,  # type: ignore[no-any-return]
+    )
+    def test_repr(
+        self, repr_: Callable[[object], str], store: TrustStore
+    ) -> None:
         assert isinstance(repr_(store), str)
 
-    def test_eq(self, store):
+    def test_eq(self, store: TrustStore) -> None:
         other = TrustStore(store)
         assert store is not other
         assert store == other
 
-    def test_bool(self, store):
+    def test_bool(self, store: TrustStore) -> None:
         assert not TrustStore()
         assert store
 
-    def test_len(self, store):
+    def test_len(self, store: TrustStore) -> None:
         assert len(store) != 0
 
-    def test_iter(self, store):
+    def test_iter(self, store: TrustStore) -> None:
         assert store[0] != store[1]
         for n, crt in enumerate(store, start=1):
             assert crt in store
         assert n == len(store)
 
-    def test_add_existing_certificate(self, store):
+    def test_add_existing_certificate(self, store: TrustStore) -> None:
         length = len(store)
         store.add(store[0])
         assert len(store) == length
 
-    def test_add_new_certificate(self, store):
+    def test_add_new_certificate(self, store: TrustStore) -> None:
         root_ca = make_root_ca()[0]
         length = len(store)
         store.add(root_ca)
@@ -321,39 +375,53 @@ class TestTrustStore:
 
 
 class TestDTLSCookie:
-    @pytest.fixture
-    def cookie(self):
+    @pytest.fixture()
+    def cookie(self) -> DTLSCookie:
         return DTLSCookie()
 
-    def test_generate_does_not_raise(self, cookie):
+    def test_generate_does_not_raise(self, cookie: DTLSCookie) -> None:
         cookie.generate()
 
-    def test_timeout(self, cookie):
+    def test_timeout(self, cookie: DTLSCookie) -> None:
         assert cookie.timeout == 60
         cookie.timeout = 1000
         assert cookie.timeout == 1000
 
 
-class _BaseConfiguration:
-    @pytest.fixture
-    def conf(self):
-        raise NotImplementedError
+class TestConfiguration:
+    @pytest.fixture(params=[DTLSConfiguration, TLSConfiguration])
+    def conf(self, request: Any) -> Union[TLSConfiguration, DTLSConfiguration]:
+        conf = request.param()
+        assert isinstance(conf, TLSConfiguration) or isinstance(
+            conf, DTLSConfiguration
+        )
+        return conf
 
-    @pytest.fixture
-    def version(self):
-        raise NotImplementedError
-
-    @pytest.mark.parametrize("repr_", (repr, str), ids=lambda f: f.__name__)
-    def test_repr(self, repr_, conf):
+    @pytest.mark.parametrize(
+        "repr_",
+        [repr, str],
+        ids=lambda f: f.__name__,  # type: ignore[no-any-return]
+    )
+    def test_repr(
+        self,
+        repr_: Callable[[object], str],
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+    ) -> None:
         assert isinstance(repr_(conf), str)
 
     @pytest.mark.parametrize("validate", [True, False])
-    def test_set_validate_certificates(self, conf, validate):
+    def test_set_validate_certificates(
+        self, conf: Union[TLSConfiguration, DTLSConfiguration], validate: bool
+    ) -> None:
         conf_ = conf.update(validate_certificates=validate)
         assert conf_.validate_certificates is validate
 
     @pytest.mark.parametrize("chain", [((), None), None])
-    def test_set_certificate_chain(self, conf, chain):
+    def test_set_certificate_chain(
+        self,
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+        chain: Tuple[Tuple[CRT, ...], _Key],
+    ) -> None:
         root_crt, root_key = make_root_ca()
         ee_crt, ee_key = make_crt(root_crt, root_key)
         if chain is None:
@@ -361,8 +429,10 @@ class _BaseConfiguration:
         conf_ = conf.update(certificate_chain=chain)
         assert conf_.certificate_chain == chain
 
-    @pytest.mark.parametrize("ciphers", (ciphers_available(),))
-    def test_set_ciphers(self, conf, ciphers):
+    def test_set_ciphers(
+        self, conf: Union[TLSConfiguration, DTLSConfiguration]
+    ) -> None:
+        ciphers = tuple(ciphers_available())
         conf_ = conf.update(ciphers=ciphers)
         assert conf_.ciphers == ciphers
 
@@ -370,32 +440,40 @@ class _BaseConfiguration:
         "inner_protocols",
         [[], (), [NextProtocol.H2, NextProtocol.H2C], [b"h2", b"h2c", b"ftp"]],
     )
-    def test_set_inner_protocols(self, conf, inner_protocols):
+    def test_set_inner_protocols(
+        self,
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+        inner_protocols: Sequence[Union[NextProtocol, bytes]],
+    ) -> None:
         conf_ = conf.update(inner_protocols=inner_protocols)
         assert conf_.inner_protocols == tuple(
             NextProtocol(_) for _ in inner_protocols
         )
 
-    def test_lowest_supported_version(self, conf, version):
-        conf_ = conf.update(lowest_supported_version=version)
-        assert conf_.lowest_supported_version is version
-
-    def test_highest_supported_version(self, conf, version):
-        conf_ = conf.update(highest_supported_version=version)
-        assert conf_.highest_supported_version is version
-
     @pytest.mark.parametrize("store", [TrustStore.system()])
-    def test_trust_store(self, conf, store):
+    def test_trust_store(
+        self,
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+        store: TrustStore,
+    ) -> None:
         conf_ = conf.update(trust_store=store)
         assert store
         assert conf_.trust_store == store
 
     @pytest.mark.parametrize("callback", [None])
-    def test_set_sni_callback(self, conf, callback):
+    def test_set_sni_callback(
+        self,
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+        callback: object,
+    ) -> None:
         assert conf.sni_callback is None
 
     @pytest.mark.parametrize("psk", [None, ("client", b"the secret key")])
-    def test_psk(self, conf, psk):
+    def test_psk(
+        self,
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+        psk: Tuple[str, bytes],
+    ) -> None:
         assert conf.pre_shared_key is None
         conf_ = conf.update(pre_shared_key=psk)
         assert conf_.pre_shared_key == psk
@@ -403,41 +481,72 @@ class _BaseConfiguration:
     @pytest.mark.parametrize(
         "psk_store", [None, {"client": b"the secret key"}]
     )
-    def test_psk_store(self, conf, psk_store):
+    def test_psk_store(
+        self,
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+        psk_store: Mapping[str, bytes],
+    ) -> None:
         assert conf.pre_shared_key_store is None
         conf_ = conf.update(pre_shared_key_store=psk_store)
         assert conf_.pre_shared_key_store == psk_store
 
 
-class TestTLSConfiguration(_BaseConfiguration):
-    @pytest.fixture
-    def conf(self):
+class TestTLSConfiguration:
+    @pytest.fixture()
+    def conf(self) -> TLSConfiguration:
         return TLSConfiguration()
 
-    @pytest.fixture(params=TLSVersion)
-    def version(self, request):
-        return request.param
+    @pytest.mark.parametrize("version", TLSVersion)
+    def test_lowest_supported_version(
+        self, conf: TLSConfiguration, version: TLSVersion
+    ) -> None:
+        conf_ = conf.update(lowest_supported_version=version)
+        assert conf_.lowest_supported_version is version
+
+    @pytest.mark.parametrize("version", TLSVersion)
+    def test_highest_supported_version(
+        self, conf: TLSConfiguration, version: TLSVersion
+    ) -> None:
+        conf_ = conf.update(highest_supported_version=version)
+        assert conf_.highest_supported_version is version
 
 
-class TestDTLSConfiguration(_BaseConfiguration):
-    @pytest.fixture
-    def conf(self):
+class TestDTLSConfiguration:
+    @pytest.fixture()
+    def conf(self) -> DTLSConfiguration:
         return DTLSConfiguration()
 
-    @pytest.fixture(params=DTLSVersion)
-    def version(self, request):
-        return request.param
+    @pytest.mark.parametrize("version", DTLSVersion)
+    def test_lowest_supported_version(
+        self, conf: DTLSConfiguration, version: DTLSVersion
+    ) -> None:
+        conf_ = conf.update(lowest_supported_version=version)
+        assert conf_.lowest_supported_version is version
+
+    @pytest.mark.parametrize("version", DTLSVersion)
+    def test_highest_supported_version(
+        self, conf: DTLSConfiguration, version: DTLSVersion
+    ) -> None:
+        conf_ = conf.update(highest_supported_version=version)
+        assert conf_.highest_supported_version is version
 
     @pytest.mark.parametrize("anti_replay", [True, False])
-    def test_set_anti_replay(self, conf, anti_replay):
+    def test_set_anti_replay(
+        self, conf: DTLSConfiguration, anti_replay: bool
+    ) -> None:
         assert conf.anti_replay is True
         conf_ = conf.update(anti_replay=anti_replay)
         assert conf_.anti_replay is anti_replay
 
     @pytest.mark.parametrize(
-        "hs_min, hs_max", [(1, 60), (42, 69), (4.2, 6.9), (42.0, 69.0)]
+        "hs_min_max", [(1, 60), (42, 69), (4.2, 6.9), (42.0, 69.0)]
     )
-    def test_handshake_timeout_minmax(self, conf, hs_min, hs_max):
+    def test_handshake_timeout_minmax(
+        self,
+        conf: DTLSConfiguration,
+        hs_min_max: Tuple[Optional[int], Optional[int]],
+    ) -> None:
+        hs_min, hs_max = hs_min_max
         assert conf.handshake_timeout_min == 1.0
         assert conf.handshake_timeout_max == 60.0
         conf_ = conf.update(
@@ -448,9 +557,14 @@ class TestDTLSConfiguration(_BaseConfiguration):
         assert conf_.handshake_timeout_max == hs_max
 
     @pytest.mark.parametrize(
-        "hs_min, hs_max", [(None, None), (1, None), (None, 60)]
+        "hs_min_max", [(None, None), (1, None), (None, 60)]
     )
-    def test_handshake_timeout_default(self, conf, hs_min, hs_max):
+    def test_handshake_timeout_default(
+        self,
+        conf: DTLSConfiguration,
+        hs_min_max: Tuple[Optional[int], Optional[int]],
+    ) -> None:
+        hs_min, hs_max = hs_min_max
         conf_ = conf.update(
             handshake_timeout_min=hs_min,
             handshake_timeout_max=hs_max,
@@ -459,60 +573,40 @@ class TestDTLSConfiguration(_BaseConfiguration):
         assert conf_.handshake_timeout_max == hs_max or 60.0
 
 
-class TestBaseContext:
+class TestContext:
     @pytest.fixture(params=[Purpose.SERVER_AUTH, Purpose.CLIENT_AUTH])
-    def purpose(self, request):
+    def purpose(self, request: Any) -> Purpose:
+        assert isinstance(request.param, Purpose)
         return request.param
 
     @pytest.fixture(params=[TLSConfiguration, DTLSConfiguration])
-    def conf(self, request):
-        return request.param()
+    def conf(self, request: Any) -> Union[TLSConfiguration, DTLSConfiguration]:
+        conf = request.param()
+        assert isinstance(conf, TLSConfiguration) or isinstance(
+            conf, DTLSConfiguration
+        )
+        return conf
 
     @pytest.fixture(params=[ServerContext, ClientContext])
-    def context(self, conf, request):
-        cls = request.param
-        return cls(conf)
+    def context(
+        self, conf: Union[TLSConfiguration, DTLSConfiguration], request: Any
+    ) -> Union[ServerContext, ClientContext]:
+        ctx = request.param(conf)
+        assert isinstance(ctx, ServerContext) or isinstance(ctx, ClientContext)
+        return ctx
 
-    def test_repr(self, context):
+    def test_repr(self, context: Union[ServerContext, ClientContext]) -> None:
         assert isinstance(repr(context), str)
 
-    def test_get_configuration(self, context, conf):
+    def test_get_configuration(
+        self,
+        context: Union[ServerContext, ClientContext],
+        conf: Union[TLSConfiguration, DTLSConfiguration],
+    ) -> None:
         assert conf
         assert context.configuration is conf
 
 
-class TestClientContext(TestBaseContext):
-    @pytest.fixture(params=[None, "hostname", "localhost"])
-    def hostname(self, request):
-        return request.param
-
-    @pytest.fixture
-    def context(self, conf, hostname):
-        return ClientContext(conf)
-
-    def test_context(self, context):
-        assert isinstance(context, ClientContext)
-
-    def test_hostname(self, context, hostname):
-        wrapped_buffers = context.wrap_buffers(hostname)
-        assert wrapped_buffers._server_hostname == hostname
-
-    def test_wrap_buffers(self, context):
-        assert isinstance(context.wrap_buffers(None), TLSWrappedBuffer)
-
-
-class TestServerContext(TestBaseContext):
-    @pytest.fixture
-    def context(self, conf):
-        return ServerContext(conf)
-
-    def test_context(self, context):
-        assert isinstance(context, ServerContext)
-
-    def test_wrap_buffers(self, context):
-        assert isinstance(context.wrap_buffers(), TLSWrappedBuffer)
-
-
 CLIENT_HELLO = (HandshakeStep.CLIENT_HELLO,)
 SERVER_HELLO = (
     HandshakeStep.SERVER_HELLO,
@@ -539,15 +633,22 @@ HANDSHAKE_OVER = (
 )
 
 
-def make_server(conf):
+def make_server(
+    conf: Union[TLSConfiguration, DTLSConfiguration]
+) -> TLSWrappedBuffer:
     return ServerContext(conf).wrap_buffers()
 
 
-def make_client(conf, hostname):
+def make_client(
+    conf: Union[TLSConfiguration, DTLSConfiguration],
+    hostname: Optional[_HostName],
+) -> TLSWrappedBuffer:
     return ClientContext(conf).wrap_buffers(hostname)
 
 
-def do_io(*, src, dst, amt=1024):
+def do_io(
+    *, src: TLSWrappedBuffer, dst: TLSWrappedBuffer, amt: int = 1024
+) -> None:
     __tracebackhide__ = True
     assert src._output_buffer, "nothing to do"
     while src._output_buffer:
@@ -556,13 +657,17 @@ def do_io(*, src, dst, amt=1024):
         dst.receive_from_network(in_transit)
 
 
-def do_send(data, *, src, dst):
+def do_send(
+    data: bytes, *, src: TLSWrappedBuffer, dst: TLSWrappedBuffer
+) -> bytes:
     amt = src.write(data)
     do_io(src=src, dst=dst)
     return dst.read(amt)
 
 
-def do_handshake(end, states):
+def do_handshake(
+    end: TLSWrappedBuffer, states: Sequence[HandshakeStep]
+) -> None:
     __tracebackhide__ = True
     while end._handshake_state is not states[0]:
         # The backend goes through every state for both
@@ -582,7 +687,9 @@ def do_handshake(end, states):
             end.do_handshake()
 
 
-def make_full_handshake(*, client, server):
+def make_full_handshake(
+    *, client: TLSWrappedBuffer, server: TLSWrappedBuffer
+) -> None:
     do_handshake(client, CLIENT_HELLO)
     do_io(src=client, dst=server)
 
@@ -602,7 +709,9 @@ def make_full_handshake(*, client, server):
     assert client.cipher() == server.cipher()
 
 
-def make_hello_verify_request(*, client, server, cookie):
+def make_hello_verify_request(
+    *, client: TLSWrappedBuffer, server: TLSWrappedBuffer, cookie: bytes
+) -> None:
     do_handshake(client, CLIENT_HELLO)
     do_io(src=client, dst=server)
     server.setcookieparam(cookie)
@@ -620,7 +729,7 @@ def make_hello_verify_request(*, client, server, cookie):
     do_io(src=server, dst=client)
 
 
-def do_communicate(args):
+def do_communicate(args: Any) -> str:
     while True:
         with subprocess.Popen(
             args,
@@ -638,18 +747,22 @@ def do_communicate(args):
 
 class TestTLSHandshake:
     @pytest.fixture(scope="class")
-    def hostname(self):
+    def hostname(self) -> _HostName:
         return "www.example.com"
 
     @pytest.fixture(scope="class")
-    def certificate_chain(self, hostname):
+    def certificate_chain(
+        self, hostname: _HostName
+    ) -> Tuple[Tuple[CRT, ...], _Key]:
         root_crt, root_key = make_root_ca()
         ee_crt, ee_key = make_crt(
             root_crt, root_key, subject=f"OU=test, CN={hostname}"
         )
         return (ee_crt, root_crt), ee_key
 
-    def test_cert_without_validation(self, certificate_chain):
+    def test_cert_without_validation(
+        self, certificate_chain: Tuple[Tuple[CRT, ...], _Key]
+    ) -> None:
         server = make_server(
             TLSConfiguration(
                 certificate_chain=certificate_chain,
@@ -665,8 +778,13 @@ class TestTLSHandshake:
         assert do_send(secret, src=client, dst=server) == secret
         assert do_send(secret, src=server, dst=client) == secret
 
-    def test_cert_with_validation(self, hostname, certificate_chain):
+    def test_cert_with_validation(
+        self,
+        hostname: _HostName,
+        certificate_chain: Tuple[Tuple[CRT, ...], _Key],
+    ) -> None:
         trust_store = TrustStore()
+        crt: CRT
         for crt in certificate_chain[0][1:]:
             trust_store.add(crt)
         server = make_server(
@@ -685,7 +803,7 @@ class TestTLSHandshake:
         assert do_send(secret, src=client, dst=server) == secret
         assert do_send(secret, src=server, dst=client) == secret
 
-    def test_psk(self):
+    def test_psk(self) -> None:
         psk = ("cli", b"secret")
         server = make_server(
             TLSConfiguration(
@@ -708,7 +826,7 @@ class TestTLSHandshake:
 
 
 class TestDTLSHandshake:
-    def test_psk(self):
+    def test_psk(self) -> None:
         psk = ("cli", b"secret")
         server = make_server(
             DTLSConfiguration(
@@ -732,7 +850,7 @@ class TestDTLSHandshake:
         assert do_send(secret, src=client, dst=server) == secret
         assert do_send(secret, src=server, dst=client) == secret
 
-    def test_resume_from_pickle(self):
+    def test_resume_from_pickle(self) -> None:
         psk = ("cli", b"secret")
         server = make_server(
             DTLSConfiguration(
@@ -765,8 +883,8 @@ class TestDTLSHandshake:
 
 @pytest.mark.skipif(sys.platform == "win32", reason="Flaky under Windows")
 class TestProgramsTLS:
-    @pytest.fixture
-    def port(self):
+    @pytest.fixture()
+    def port(self) -> int:
         """Return a free port
 
         Note:
@@ -779,8 +897,10 @@ class TestProgramsTLS:
             port = sock.getsockname()[1]
         return port
 
-    @pytest.fixture
-    def server(self, rootpath, port):
+    @pytest.fixture()
+    def server(
+        self, rootpath: Path, port: int
+    ) -> Iterator[subprocess.Popen[str]]:
         args = [
             sys.executable,
             str(rootpath / "programs" / "server.py"),
@@ -797,7 +917,7 @@ class TestProgramsTLS:
 
     @pytest.mark.usefixtures("server")
     @pytest.mark.timeout(10)
-    def test_communicate(self, rootpath, port):
+    def test_communicate(self, rootpath: Path, port: int) -> None:
         secret = "a very secret message"
         args = [
             sys.executable,
@@ -815,8 +935,8 @@ class TestProgramsTLS:
 
 @pytest.mark.skipif(sys.platform == "win32", reason="Flaky under Windows")
 class TestProgramsDTLS:
-    @pytest.fixture
-    def port(self):
+    @pytest.fixture()
+    def port(self) -> int:
         """Return a free port
 
         Note:
@@ -829,8 +949,10 @@ class TestProgramsDTLS:
             port = sock.getsockname()[1]
         return port
 
-    @pytest.fixture
-    def server(self, rootpath, port):
+    @pytest.fixture()
+    def server(
+        self, rootpath: Path, port: int
+    ) -> Iterator[subprocess.Popen[str]]:
         args = [
             sys.executable,
             str(rootpath / "programs" / "server.py"),
@@ -847,7 +969,7 @@ class TestProgramsDTLS:
 
     @pytest.mark.usefixtures("server")
     @pytest.mark.timeout(10)
-    def test_communication(self, rootpath, port):
+    def test_communication(self, rootpath: Path, port: int) -> None:
         secret = "a very secret message"
         args = [
             sys.executable,
